@@ -16,6 +16,7 @@
 # along with Ansible.  If not, see <http://www.gnu.org/licenses/>.
 
 import sys
+import re
 import os
 import shlex
 import yaml
@@ -24,8 +25,8 @@ import optparse
 import operator
 from ansible import errors
 from ansible import __version__
-from ansible.utils.template import *
 from ansible.utils.plugins import *
+from ansible.utils import template
 import ansible.constants as C
 import time
 import StringIO
@@ -36,6 +37,8 @@ import pipes
 import random
 import difflib
 import warnings
+import traceback
+import getpass
 
 VERBOSITY=0
 
@@ -67,14 +70,15 @@ except ImportError:
     pass
 
 ###############################################################
-# abtractions around keyczar
+# Abstractions around keyczar
+###############################################################
 
 def key_for_hostname(hostname):
     # fireball mode is an implementation of ansible firing up zeromq via SSH
     # to use no persistent daemons or key management
 
     if not KEYCZAR_AVAILABLE:
-        raise errors.AnsibleError("python-keyczar must be installed to use fireball mode")
+        raise errors.AnsibleError("python-keyczar must be installed to use fireball/accelerated mode")
 
     key_path = os.path.expanduser("~/.fireball.keys")
     if not os.path.exists(key_path):
@@ -124,6 +128,9 @@ def jsonify(result, format=False):
     if result is None:
         return "{}"
     result2 = result.copy()
+    for key, value in result2.items():
+        if type(value) is str:
+            result2[key] = value.decode('utf-8', 'ignore')
     if format:
         return json.dumps(result2, sort_keys=True, indent=4)
     else:
@@ -149,21 +156,51 @@ def is_changed(result):
 
     return (result.get('changed', False) in [ True, 'True', 'true'])
 
-def check_conditional(conditional):
+def check_conditional(conditional, basedir, inject, fail_on_undefined=False, jinja2=False):
+
+    if jinja2:
+        conditional = "jinja2_compare %s" % conditional
+
+    if conditional.startswith("jinja2_compare"):
+        conditional = conditional.replace("jinja2_compare ","")
+        # allow variable names
+        if conditional in inject and str(inject[conditional]).find('-') == -1:
+            conditional = inject[conditional]
+        conditional = template.template(basedir, conditional, inject, fail_on_undefined=fail_on_undefined)
+        # a Jinja2 evaluation that results in something Python can eval!
+        presented = "{%% if %s %%} True {%% else %%} False {%% endif %%}" % conditional
+        conditional = template.template(basedir, presented, inject)
+        val = conditional.strip()
+        if val == presented:
+            # the templating failed, meaning most likely a 
+            # variable was undefined. If we happened to be 
+            # looking for an undefined variable, return True,
+            # otherwise fail
+            if conditional.find("is undefined") != -1:
+                return True
+            elif conditional.find("is defined") != -1:
+                return False
+            else:
+                raise errors.AnsibleError("error while evaluating conditional: %s" % conditional)
+        elif val == "True":
+            return True
+        elif val == "False":
+            return False
+        else:
+            raise errors.AnsibleError("unable to evaluate conditional: %s" % conditional)
 
     if not isinstance(conditional, basestring):
         return conditional
 
-    def is_set(var):
-        return not var.startswith("$")
-
-    def is_unset(var):
-        return var.startswith("$")
-
     try:
-        return eval(conditional.replace("\n", "\\n"))
+        conditional = conditional.replace("\n", "\\n")
+        result = safe_eval(conditional)
+        if result not in [ True, False ]:
+            raise errors.AnsibleError("Conditional expression must evaluate to True or False: %s" % conditional)
+        return result
+
     except (NameError, SyntaxError):
-        raise errors.AnsibleError("Could not evaluate the expression: " + conditional)
+        raise errors.AnsibleError("Could not evaluate the expression: (%s)" % conditional)
 
 def is_executable(path):
     '''is the given path executable?'''
@@ -171,18 +208,34 @@ def is_executable(path):
             or stat.S_IXGRP & os.stat(path)[stat.ST_MODE]
             or stat.S_IXOTH & os.stat(path)[stat.ST_MODE])
 
-def prepare_writeable_dir(tree):
+def unfrackpath(path):
+    ''' 
+    returns a path that is free of symlinks, environment
+    variables, relative path traversals and symbols (~)
+    example:
+    '$HOME/../../var/mail' becomes '/var/spool/mail'
+    '''
+    return os.path.normpath(os.path.realpath(os.path.expandvars(os.path.expanduser(path))))
+
+def prepare_writeable_dir(tree,mode=0777):
     ''' make sure a directory exists and is writeable '''
 
-    if tree != '/':
-        tree = os.path.realpath(os.path.expanduser(tree))
+    # modify the mode to ensure the owner at least
+    # has read/write access to this directory
+    mode |= 0700
+
+    # make sure the tree path is always expanded
+    # and normalized and free of symlinks
+    tree = unfrackpath(tree)
+
     if not os.path.exists(tree):
         try:
-            os.makedirs(tree)
+            os.makedirs(tree, mode)
         except (IOError, OSError), e:
-            exit("Could not make dir %s: %s" % (tree, e))
+            raise errors.AnsibleError("Could not make dir %s: %s" % (tree, e))
     if not os.access(tree, os.W_OK):
-        exit("Cannot write to path %s" % tree)
+        raise errors.AnsibleError("Cannot write to path %s" % tree)
+    return tree
 
 def path_dwim(basedir, given):
     '''
@@ -190,11 +243,27 @@ def path_dwim(basedir, given):
     '''
 
     if given.startswith("/"):
-        return given
+        return os.path.abspath(given)
     elif given.startswith("~"):
-        return os.path.expanduser(given)
+        return os.path.abspath(os.path.expanduser(given))
     else:
-        return os.path.join(basedir, given)
+        return os.path.abspath(os.path.join(basedir, given))
+
+def path_dwim_relative(original, dirname, source, playbook_base, check=True):
+    ''' find one file in a directory one level up in a dir named dirname relative to current '''
+    # (used by roles code)
+
+    basedir = os.path.dirname(original)
+    template2 = os.path.join(basedir, '..', dirname, source)
+    source2 = path_dwim(basedir, template2)
+    if os.path.exists(source2):
+        return source2
+    obvious_local_path = path_dwim(playbook_base, source)
+    if os.path.exists(obvious_local_path):
+        return obvious_local_path
+    if check:
+        raise errors.AnsibleError("input file not found at %s or %s" % (source2, obvious_local_path))
+    return source2 # which does not exist
 
 def json_loads(data):
     ''' parse a JSON string and return a data structure '''
@@ -237,9 +306,31 @@ def parse_json(raw_data):
             return { "failed" : True, "parsed" : False, "msg" : orig_data }
         return results
 
+def smush_braces(data):
+    ''' smush Jinaj2 braces so unresolved templates like {{ foo }} don't get parsed weird by key=value code '''
+    while data.find('{{ ') != -1:
+        data = data.replace('{{ ', '{{')
+    while data.find(' }}') != -1:
+        data = data.replace(' }}', '}}')
+    return data
+
+def smush_ds(data):
+    # things like key={{ foo }} are not handled by shlex.split well, so preprocess any YAML we load
+    # so we do not have to call smush elsewhere
+    if type(data) == list:
+        return [ smush_ds(x) for x in data ]
+    elif type(data) == dict:
+        for (k,v) in data.items():
+            data[k] = smush_ds(v)
+        return data
+    elif isinstance(data, basestring):
+        return smush_braces(data)
+    else:
+        return data
+
 def parse_yaml(data):
     ''' convert a yaml string to a data structure '''
-    return yaml.safe_load(data)
+    return smush_ds(yaml.safe_load(data))
 
 def process_yaml_error(exc, data, path=None):
     if hasattr(exc, 'problem_mark'):
@@ -279,11 +370,12 @@ def parse_yaml_from_file(path):
 
 def parse_kv(args):
     ''' convert a string of key/value items to a dict '''
-
     options = {}
     if args is not None:
         # attempting to split a unicode here does bad things
-        vargs = shlex.split(str(args), posix=True)
+        args = args.encode('utf-8')
+        vargs = [x.decode('utf-8') for x in shlex.split(args, posix=True)]
+        #vargs = shlex.split(str(args), posix=True)
         for x in vargs:
             if x.find("=") != -1:
                 k, v = x.split("=",1)
@@ -291,30 +383,30 @@ def parse_kv(args):
     return options
 
 def merge_hash(a, b):
-    ''' merges hash b into a
-    this means that if b has key k, the resulting has will have a key k
-    which value comes from b
-    said differently, all key/value combination from b will override a's '''
+    ''' recursively merges hash b into a
+    keys from b take precedence over keys from a '''
 
-    # and iterate over b keys
+    result = copy.deepcopy(a)
+
+    # next, iterate over b keys and values
     for k, v in b.iteritems():
-        if k in a and isinstance(a[k], dict):
-            # if this key is a hash and exists in a
-            # we recursively call ourselves with 
-            # the key value of b
-            a[k] = merge_hash(a[k], v)
+        # if there's already such key in a
+        # and that key contains dict
+        if k in result and isinstance(result[k], dict):
+            # merge those dicts recursively
+            result[k] = merge_hash(a[k], v)
         else:
-            # k is not in a, no need to merge b, we just deecopy
-            # or k is not a dictionnary, no need to merge b either, we just deecopy it
-            a[k] = v
-    # finally, return the resulting hash when we're done iterating keys
-    return a
+            # otherwise, just copy a value from b to a
+            result[k] = v
+
+    return result
 
 def md5s(data):
     ''' Return MD5 hex digest of data. '''
 
+    buf = StringIO.StringIO(data)
     digest = _md5()
-    digest.update(data.encode('utf-8'))
+    digest.update(buf.read())
     return digest.hexdigest()
 
 def md5(filename):
@@ -413,7 +505,7 @@ def base_parser(constants=C, usage="", output_opts=False, runas_opts=False,
 
     parser = SortedOptParser(usage, version=version("%prog"))
     parser.add_option('-v','--verbose', default=False, action="callback",
-        callback=increment_debug, help="verbose mode (-vvv for more)")
+        callback=increment_debug, help="verbose mode (-vvv for more, -vvvv to enable connection debugging)")
 
     parser.add_option('-f','--forks', dest='forks', default=constants.DEFAULT_FORKS, type='int',
         help="specify number of parallel processes to use (default=%s)" % constants.DEFAULT_FORKS)
@@ -426,6 +518,8 @@ def base_parser(constants=C, usage="", output_opts=False, runas_opts=False,
         help='use this file to authenticate the connection')
     parser.add_option('-K', '--ask-sudo-pass', default=False, dest='ask_sudo_pass', action='store_true',
         help='ask for sudo password')
+    parser.add_option('--list-hosts', dest='listhosts', action='store_true',
+        help='outputs a list of matching hosts; does not execute anything else')
     parser.add_option('-M', '--module-path', dest='module_path',
         help="specify path(s) to module library (default=%s)" % constants.DEFAULT_MODULE_PATH,
         default=None)
@@ -445,7 +539,7 @@ def base_parser(constants=C, usage="", output_opts=False, runas_opts=False,
             help='log output to this directory')
 
     if runas_opts:
-        parser.add_option("-s", "--sudo", default=False, action="store_true",
+        parser.add_option("-s", "--sudo", default=constants.DEFAULT_SUDO, action="store_true",
             dest='sudo', help="run operations with sudo (nopasswd)")
         parser.add_option('-U', '--sudo-user', dest='sudo_user', help='desired sudo user (default=root)',
             default=None)   # Can't default to root because we need to detect when this option was given
@@ -467,17 +561,33 @@ def base_parser(constants=C, usage="", output_opts=False, runas_opts=False,
 
     if check_opts:
         parser.add_option("-C", "--check", default=False, dest='check', action='store_true',
-            help="don't make any changes, instead try to predict some of the changes that may occur"
+            help="don't make any changes; instead, try to predict some of the changes that may occur"
         )
 
     if diff_opts:
         parser.add_option("-D", "--diff", default=False, dest='diff', action='store_true',
-            help="when changing (small) files and templates, show the differences in those files, works great with --check"
+            help="when changing (small) files and templates, show the differences in those files; works great with --check"
         )
 
 
     return parser
 
+def ask_passwords(ask_pass=False, ask_sudo_pass=False):
+    sshpass = None
+    sudopass = None
+    sudo_prompt = "sudo password: "
+
+    if ask_pass:
+        sshpass = getpass.getpass(prompt="SSH password: ")
+        sudo_prompt = "sudo password [defaults to SSH password]: "
+
+    if ask_sudo_pass:
+        sudopass = getpass.getpass(prompt=sudo_prompt)
+        if ask_pass and sudopass == '':
+            sudopass = sshpass
+
+    return (sshpass, sudopass)
+
 def do_encrypt(result, encrypt, salt_size=None, salt=None):
     if PASSLIB_AVAILABLE:
         try:
@@ -544,6 +654,7 @@ def compile_when_to_only_if(expression):
     # when: int $x in $alist
     # when: float $x > 2 and $y <= $z
     # when: str $x != $y
+    # when: jinja2_compare asdf  # implies {{ asdf }}
 
     if type(expression) not in [ str, unicode ]:
         raise errors.AnsibleError("invalid usage of when_ operator: %s" % expression)
@@ -561,8 +672,6 @@ def compile_when_to_only_if(expression):
                 tcopy[i] = t
         return " ".join(tcopy)
 
-
-
     # when_failed / when_changed
     elif tokens[0] in [ 'failed', 'changed' ]:
         tcopy = tokens[1:]
@@ -573,8 +682,6 @@ def compile_when_to_only_if(expression):
                 tcopy[i] = t
         return " ".join(tcopy)
 
-
-
     # when_integer / when_float / when_string
     elif tokens[0] in [ 'integer', 'float', 'string' ]:
         cast = None
@@ -586,12 +693,18 @@ def compile_when_to_only_if(expression):
             cast = 'float'
         tcopy = tokens[1:]
         for (i,t) in enumerate(tokens[1:]):
-            if t.find("$") != -1:
-                # final variable substitution will happen in Runner code
-                tcopy[i] = "%s('''%s''')" % (cast, t)
+            #if re.search(t, r"^\w"):
+                # bare word will turn into Jinja2 so all the above
+                # casting is really not needed
+                #tcopy[i] = "%s('''%s''')" % (cast, t)
+            t2 = t.strip()
+            if (t2[0].isalpha() or t2[0] == '$') and cast == 'str' and t2 != 'in':
+                tcopy[i] = "'%s'" % (t)
             else:
                 tcopy[i] = t
-        return " ".join(tcopy)
+        result = " ".join(tcopy)
+        return result
+
 
     # when_boolean
     elif tokens[0] in [ 'bool', 'boolean' ]:
@@ -601,6 +714,9 @@ def compile_when_to_only_if(expression):
                 tcopy[i] = "(is_set('''%s''') and '''%s'''.lower() not in ('false', 'no', 'n', 'none', '0', ''))" % (t, t)
         return " ".join(tcopy)
 
+    # the stock 'when' without qualification (new in 1.2), assumes Jinja2 terms
+    elif tokens[0] == 'jinja2_compare':
+        return " ".join(tokens)
     else:
         raise errors.AnsibleError("invalid usage of when_ operator: %s" % expression)
 
@@ -622,6 +738,13 @@ def make_sudo_cmd(sudo_user, executable, cmd):
         prompt, sudo_user, executable or '$SHELL', pipes.quote(cmd))
     return ('/bin/sh -c ' + pipes.quote(sudocmd), prompt)
 
+_TO_UNICODE_TYPES = (unicode, type(None))
+
+def to_unicode(value):
+    if isinstance(value, _TO_UNICODE_TYPES):
+        return value
+    return value.decode("utf-8")
+
 def get_diff(diff):
     # called by --diff usage in playbook and runner via callbacks
     # include names in diffs 'before' and 'after' and do diff -U 10
@@ -647,16 +770,86 @@ def get_diff(diff):
                     after_header = "after: %s" % diff['after_header']
                 else:
                     after_header = 'after'
-                differ = difflib.unified_diff(diff['before'].splitlines(True), diff['after'].splitlines(True), before_header, after_header, '', '', 10)
+                differ = difflib.unified_diff(to_unicode(diff['before']).splitlines(True), to_unicode(diff['after']).splitlines(True), before_header, after_header, '', '', 10)
                 for line in list(differ):
                     ret.append(line)
-            return "".join(ret)
+            return u"".join(ret)
     except UnicodeDecodeError:
         return ">> the files are different, but the diff library cannot compare unicode strings"
 
 def is_list_of_strings(items):
-   for x in items: 
-       if not isinstance(x, basestring):
-           return False
-   return True
+    for x in items:
+        if not isinstance(x, basestring):
+            return False
+    return True
+
+def safe_eval(str):
+    '''
+    this is intended for allowing things like:
+    with_items: {{ a_list_variable }}
+    where Jinja2 would return a string
+    but we do not want to allow it to call functions (outside of Jinja2, where
+    the env is constrained)
+    '''
+    # FIXME: is there a more native way to do this?
+
+    def is_set(var):
+        return not var.startswith("$") and not '{{' in var
+
+    def is_unset(var):
+        return var.startswith("$") or '{{' in var
+
+    # do not allow method calls to modules
+    if not isinstance(str, basestring):
+        # already templated to a datastructure, perhaps?
+        return str
+    if re.search(r'\w\.\w+\(', str):
+        return str
+    # do not allow imports
+    if re.search(r'import \w+', str):
+        return str
+    try:
+        return eval(str)
+    except Exception, e:
+        return str
+
+
+def listify_lookup_plugin_terms(terms, basedir, inject):
+
+    if isinstance(terms, basestring):
+        # someone did:
+        #    with_items: alist
+        # OR
+        #    with_items: {{ alist }}
+
+        stripped = terms.strip()
+        if not (stripped.startswith('{') or stripped.startswith('[')) and not stripped.startswith("/"):
+            # if not already a list, get ready to evaluate with Jinja2
+            # not sure why the "/" is in above code :)
+            try:
+                new_terms = template.template(basedir, "{{ %s }}" % terms, inject)
+                if isinstance(new_terms, basestring) and new_terms.find("{{") != -1:
+                    pass
+                else:
+                    terms = new_terms
+            except:
+                pass
+
+        if '{' in terms or '[' in terms:
+            # Jinja2 already evaluated a variable to a list.
+            # Jinja2-ified list needs to be converted back to a real type
+            # TODO: something a bit less heavy than eval
+            return safe_eval(terms)
+
+        if isinstance(terms, basestring):
+            terms = [ terms ]
+
+    return terms
+
+def combine_vars(a, b):
+    if C.DEFAULT_HASH_BEHAVIOUR == "merge":
+        return merge_hash(a, b)
+    else:
+        return dict(a.items() + b.items())
+
 
